/*
	CNCPTS, CNC Program Transfer Software

	Copyright (C) 2001 Michael R. Donat, 5919 Hwy 191, Dodgeville, WI 53533
	(donat@donat.org)

	This file is part of CNCPTS.

	CNCPTS is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	CNCPTS is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with CNCPTS; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <io.h>
#include <conio.h>
#include <stdio.h>
#include <stdlib.h>
#include <dos.h>
#include <bios.h>
#include <fcntl.h>
#include <memory.h>
#include <malloc.h>
#include <math.h>
#include <string.h>

#include "serial.h"
int serialport(int commport) {
	switch(commport) {
   	case 1: return COM_1;
      case 2: return COM_2;
      case 3: return COM_3;
      case 4: return COM_4;
		default: return COM_1;
      }
   }
int serialbaud(int baudrate) {
	switch(baudrate) {
   	case 1200: return SERIAL_BAUD_1200;
      case 2400: return SERIAL_BAUD_2400;
   	case 4800: return SERIAL_BAUD_4800;
      case 9600: return SERIAL_BAUD_9600;
   	case 19200: return SERIAL_BAUD_19200;
		default: return SERIAL_BAUD_9600;
      }
   }
int serialconf(int databits,int stopbits, int parity) {
	int data,stop,par;
	switch(databits) {
   	case 5: data=SERIAL_BITS_5;
			break;
      case 6: data=SERIAL_BITS_6;
			break;
   	case 7: data=SERIAL_BITS_7;
			break;
      case 8: data=SERIAL_BITS_8;
			break;
		default: data=SERIAL_BITS_7;
      	break;
      }
	switch(stopbits) {
   	case 1: stop=SERIAL_STOP_1;
			break;
      case 2: stop=SERIAL_STOP_2;
			break;
		default: stop=SERIAL_STOP_1;
      	break;
      }
	switch(parity) {
   	case 0: par=SERIAL_PARITY_NONE;
			break;
      case 1: par=SERIAL_PARITY_ODD;
      	break;
      case 2: par=SERIAL_PARITY_EVEN;
      	break;
      default: par=SERIAL_PARITY_EVEN;
      	break;
      }
	return (data | stop | par);
   }

void (_interrupt _far *Old_Serial_ISR)();
char serial_buffer[SERIAL_BUFF_SIZE];
int serial_end   = -1;
int serial_start = -1;
int serial_ch;
int char_ready=0;
int old_int_mask;
int open_port;
int serial_lock = 0;
int serial_open=0;

void _interrupt _far serialISR(void) {
	_asm sti
	serial_ch = inp(open_port + SERIAL_RBF);
	if (++serial_end > SERIAL_BUFF_SIZE-1)
		serial_end = 0;
	serial_buffer[serial_end] = serial_ch;
	++char_ready;
	outp(PIC_ICR,0x20);
	}
int serialready(void) {
	return(char_ready);
	}
int serialread(void) {
	int ch;
	if (serial_end != serial_start) {
		if (++serial_start > SERIAL_BUFF_SIZE-1)
			serial_start = 0;
		ch = serial_buffer[serial_start];
		if (char_ready > 0)
			--char_ready;
		return(ch);
		}
	else
		return(0);
	}
void serialwrite(char ch) {
	while(!(inp(open_port + SERIAL_LSR) & 0x20)){}
	_asm cli
	outp(open_port + SERIAL_THR, ch);
	_asm sti
	}
void serialprint(char *string,int cr) {
	int index, length;
	length = strlen(string);
	for (index=0; index<length; index++)
		serialwrite(string[index]);
	if (cr)
		serialwrite(13);
	}
int serialopen(int port_base, int baud, int configuration) {
		unsigned char data;
	if(!serial_open) {
		serial_open=1;
		open_port = port_base;
		outp(port_base + SERIAL_LCR, SERIAL_DIV_LATCH_ON);
		outp(port_base + SERIAL_DLL, baud);
		outp(port_base + SERIAL_DLH, 0);
		outp(port_base + SERIAL_LCR, configuration);
		data = inp(port_base + SERIAL_MCR);
		data = SET_BITS(data,SERIAL_GP02);
		outp(port_base + SERIAL_MCR, data);
		outp(port_base + SERIAL_IER, 1);
		if (port_base == COM_1) {
			Old_Serial_ISR = _dos_getvect(INT_SERIAL_PORT_0);
			_dos_setvect(INT_SERIAL_PORT_0, serialISR);
			}
		else {
			Old_Serial_ISR = _dos_getvect(INT_SERIAL_PORT_1);
			_dos_setvect(INT_SERIAL_PORT_1, serialISR);
			}
		old_int_mask = inp(PIC_IMR);
	   switch(port_base) {
   		case COM_1:
				outp(PIC_IMR, old_int_mask & 0xEF);
   	   	break;
   		case COM_2:
				outp(PIC_IMR, old_int_mask & 0xF7);
   	   	break;
   		case COM_3:
				outp(PIC_IMR, old_int_mask & 0xEF);
   	   	break;
   		case COM_4:
				outp(PIC_IMR, old_int_mask & 0xF7);
   	   	break;
   		default:
	      	break;
   	   }
		return 1;
      }
   else return 0;
	}
int serialclose(void) {
	unsigned char data;
	if(serial_open) {
   	serial_open=0;
		data = inp(open_port + SERIAL_MCR);
		data = RESET_BITS(data,SERIAL_GP02);
		outp(open_port + SERIAL_MCR, data);
		outp(open_port + SERIAL_IER, 0);
		outp(PIC_IMR, old_int_mask );
		if (open_port == COM_1 || open_port==COM_3) {
			_dos_setvect(INT_SERIAL_PORT_0, Old_Serial_ISR);
			}
		else {
			_dos_setvect(INT_SERIAL_PORT_1, Old_Serial_ISR);
			}
		serial_end   = -1;
		serial_start = -1;
		char_ready   =  0;
		return 1;
		}
   else return 0;
	}
void timedelay(int clicks) {
	long far *clock = (long far *)0x0000046CL; // address of timer
	long start_time; // starting time
	start_time = *clock;
	while(labs(*clock - start_time) < (long)clicks){}
	}

void serialflush(void) {
	int index;
	for (index=0; index<32; index++) {
		serialread();
		timedelay(1);
		}
	}
int serialreadwait(void) {
	while(!serialready());
	return(serialread());
	}

