/*
	CNCPTS, CNC Program Transfer Software

	Copyright (C) 2001 Michael R. Donat, 5919 Hwy 191, Dodgeville, WI 53533
	(donat@donat.org)

	This file is part of CNCPTS.

	CNCPTS is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	CNCPTS is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with CNCPTS; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef _SERIAL_H
#define _SERIAL_H

#define SET_BITS(x,bits)   (x | bits)
#define RESET_BITS(x,bits) (x & ~bits)
#define DELAY_1_SECOND  18
#define DELAY_2_SECOND  38
#define DELAY_3_SECOND  48

// registers indices in 16450/16550 UART
#define SERIAL_RBF        0    // the read buffer
#define SERIAL_THR        0    // the write buffer
#define SERIAL_IER        1    // the int. enable register
#define SERIAL_IIR        2    // the int. identification register
#define SERIAL_LCR        3    // control data config. and divisor latch
#define SERIAL_MCR        4    // modem control reg.
#define SERIAL_LSR        5    // line status reg.
#define SERIAL_MSR        6    // modem status of cts, ring etc.
#define SERIAL_DLL        0    // the low byte of baud rate divisor
#define SERIAL_DLH        1    // the hi byte of divisor latch

// baud rate divisors
#define SERIAL_BAUD_1200  96     // baud rate divisors for 1200 baud - 19200
#define SERIAL_BAUD_2400  48
#define SERIAL_BAUD_4800  24
#define SERIAL_BAUD_9600  12
#define SERIAL_BAUD_19200  6

// bit masks for different comm settings
#define SERIAL_STOP_1      0     // 1 stop bit per character
#define SERIAL_STOP_2      4     // 2 stop bits per character
#define SERIAL_BITS_5      0     // send 5 bit characters
#define SERIAL_BITS_6      1     // send 6 bit characters
#define SERIAL_BITS_7      2     // send 7 bit characters
#define SERIAL_BITS_8      3     // send 8 bit characters
#define SERIAL_PARITY_NONE 0     // no parity
#define SERIAL_PARITY_ODD  8     // odd parity
#define SERIAL_PARITY_EVEN 24    // even parity
#define SERIAL_DIV_LATCH_ON 128  // used to turn reg 0,1 into divisor latch
#define SERIAL_GP02        8     // enable interrupt

// comm port I/O port base addresses
#define COM_1              0x3F8 // base port address of port 0
#define COM_2              0x2F8 // base port address of port 1
#define COM_3              0x3E8 // base port address of port 2
#define COM_4              0x2E8 // base port address of port 3

// programmable interrupt controller addresses
#define PIC_IMR    0x21   // pic's interrupt mask reg.
#define PIC_ICR    0x20   // pic's interupt control reg.
#define INT_SERIAL_PORT_0    0x0C  // port 0 interrupt com 1 & 3
#define INT_SERIAL_PORT_1    0x0B  // port 1 interrupt com 2 & 4
#define SERIAL_BUFF_SIZE 1024    // size of re-circulating receive buffer


extern void (_interrupt _far *Old_Serial_ISR)();  // holds old comm port interrupt handler
extern char serial_buffer[SERIAL_BUFF_SIZE];  // the receive buffer
extern int serial_end;       // indexes into receive buffer
extern int serial_start;
extern int serial_ch;
extern int char_ready;       // current character and ready flag
extern int old_int_mask;     // the old interrupt mask on the PIC
extern int open_port;        // the currently open port
extern int serial_lock;      // serial ISR semaphore so the buffer
                             // isn't altered will it is being written
                             // to by the ISR


void timedelay(int);

// serial communications
int serialport(int);
int serialbaud(int);
int serialconf(int,int,int);
void _interrupt _far serialISR(void);
int  serialready(void);
int  serialread(void);
int  serialreadwait(void);
void serialwrite(char ch);
void serialprint(char *string,int cr);
int  serialopen(int port_base, int baud, int configuration);
int  serialclose(void);
void serialflush(void);
#endif

